/*
 * Copyright (c) 2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <dk_buttons_and_leds.h>
#include <ncs_version.h>
#include <stdio.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/uart.h>
#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>
#include "nrf_errno.h"

#include "modem.h"
#include "tcp.h"

LOG_MODULE_REGISTER(Proxy, LOG_LEVEL_DBG);

void do_send(int socket, const char* buf, unsigned int size /*,  bool nonblock = false */) {
    unsigned int sent = 0;
    unsigned int count = 0;

    const int start_ms = k_uptime_get_32();
    while (sent < size) {
        // int n = send(socket, &buf[sent], size - sent, 0);
        int n = send(socket, &buf[sent], size - sent, MSG_DONTWAIT);

        if (n == -1) {
            switch (errno) {
                case NRF_ESHUTDOWN:
                    LOG_WRN("Socket has been shut down");
                    k_msleep(200);
                    break;
                case NRF_ECANCELED:
                    LOG_WRN("Operation has been canceled");
                    k_msleep(200);
                    break;
                case NRF_ENOMEM:
                    LOG_WRN("Insufficient memory available");
                    k_msleep(200);
                    break;
                case NRF_EPROTO:
                    LOG_WRN("Protocol error");
                    k_msleep(200);
                    break;
                case NRF_EPERM:
                case NRF_ENOENT:
                case NRF_ESRCH:
                case NRF_EINTR:
                case NRF_EIO:
                case NRF_ENXIO:
                case NRF_E2BIG:
                case NRF_ENOEXEC:
                case NRF_EBADF:
                case NRF_ECHILD:
                case NRF_EAGAIN:
                    LOG_WRN("Temporary error, retrying...");
                    k_msleep(200);
                    break;
                case NRF_EACCES:
                case NRF_EFAULT:
                case NRF_EBUSY:
                case NRF_EEXIST:
                case NRF_EXDEV:
                case NRF_ENODEV:
                case NRF_ENOTDIR:
                case NRF_EISDIR:
                case NRF_EINVAL:
                case NRF_ENFILE:
                case NRF_EMFILE:
                case NRF_ENOTTY:
                case NRF_ETXTBSY:
                case NRF_EFBIG:
                case NRF_ENOSPC:
                case NRF_ESPIPE:
                case NRF_EROFS:
                case NRF_EMLINK:
                case NRF_EPIPE:
                case NRF_EDOM:
                case NRF_ERANGE:
                case NRF_ENOMSG:
                case NRF_EDEADLK:
                case NRF_ENOLCK:
                case NRF_ENOSTR:
                case NRF_ENODATA:
                case NRF_ETIME:
                case NRF_ENOSR:
                case NRF_EBADMSG:
                case NRF_ENOSYS:
                case NRF_ENOTEMPTY:
                case NRF_ENAMETOOLONG:
                case NRF_ELOOP:
                case NRF_EOPNOTSUPP:
                case NRF_ECONNRESET:
                case NRF_ENOBUFS:
                case NRF_EAFNOSUPPORT:
                case NRF_EPROTOTYPE:
                case NRF_ENOTSOCK:
                case NRF_ENOPROTOOPT:
                case NRF_ECONNREFUSED:
                case NRF_EADDRINUSE:
                case NRF_ECONNABORTED:
                case NRF_ENETUNREACH:
                case NRF_ENETDOWN:
                case NRF_ETIMEDOUT:
                case NRF_EHOSTUNREACH:
                case NRF_EINPROGRESS:
                case NRF_EALREADY:
                case NRF_EDESTADDRREQ:
                case NRF_EMSGSIZE:
                case NRF_EPROTONOSUPPORT:
                case NRF_EADDRNOTAVAIL:
                case NRF_ENETRESET:
                case NRF_EISCONN:
                case NRF_ENOTCONN:
                case NRF_ENOTSUP:
                case NRF_EILSEQ:
                case NRF_EOVERFLOW:
                // case NRF_EWOULDBLOCK:
                case NRF_ENOTBLK:
                case NRF_EPFNOSUPPORT:
                case NRF_EHOSTDOWN:
                case NRF_ESOCKTNOSUPPORT:
                case NRF_ETOOMANYREFS:
                default:
                    LOG_ERR("Failed to send data, errno %d", errno);
                    k_msleep(200);
                    break;
            }
        } else {
            sent += n;
            LOG_INF("  --> sent a chunk of %u bytes (send no. %u, total sent = %u) - %dms\n", n, ++count, sent,
                    k_uptime_get_32() - start_ms);
        }
    }
    k_msleep(1000 - (k_uptime_get_32() - start_ms));
}

int main(void) {
    int ret;
    int err;

    err = modem_configure();
    if (err) {
        LOG_ERR("Failed to configure the modem");
        return 0;
    }

    if (server_resolve() != 0) {
        LOG_ERR("Failed to resolve server name");
        return 0;
    }

    if (server_connect() != 0) {
        LOG_ERR("Failed to initialize client");
        return 0;
    }

    const uint8_t buffer[] =
        "$VAL$2025-01-27 09:11:39.234       "
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGH"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789\r\n";

    while (1) {
        do_send(sock, buffer, sizeof(buffer));
    }

    (void)close(sock);
}